/*
  APE v.1.0. Copyright (C) Matti Pirinen
   
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  version 2 as published by the Free Software Foundation.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program (gpl.txt); if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
  02110-1301, USA.
  
  www.gnu.org/licenses/gpl.html
*/


#define UL unsigned long

/*FILENAMES:*/

/* Fixed filenames are given as macros below.*/

//info about alleles that can be inferred from the known data
#define INFERRED_OUTPUT "inferred.out"
//simpler output of APE when it succeeds
#define SIMPLE_OUTPUT "paths.out"
//full output of APE when it succeeds
#define TOTAL_OUTPUT "total.out"
//contains the components that APE_extract shall extract
#define EXTRACTIONS_FILE "extracted_component"
//APE_extract writes new datafile
#define EXTRACTED_DATA_FILE "extracted_data"
//used in drawing the pedigree with pedfiddler
#define PEDIGREE_LIST_OUTPUT "pedlist.out"
//the pedigree divided into connected components (if more than 1)
#define CONNECTED_COMPONENTS_OUTPUT "connected.out"
 //list of the components of the pedigree
#define COMPONENTS_OUTPUT "components.out"


typedef int pair[2];

struct tensor_list
{
  double proba;
  pair *tensor;
};
typedef struct tensor_list t_list;

struct children_list
{
  int nchildren;
  int *children;
};
typedef struct children_list CHILDREN;

struct parental_info
{
  int nlevels; //number of different levels at which ind is a parent
  int *levels; //list of those level
  int *index; //corresponding indexes as parent at those levels
};
typedef struct parental_info PARENTAL;

struct ordering_list
{
  int index[2];
  double proba;
  double difference;
  double score;
};
typedef struct ordering_list o_list;

/************ Component structures *******************************************************************/

struct connected 
{
  int nparents;
  int nchildren;       
  int nknowndata;      /*number of parents whose data is known in advance*/
  int ncloned;         /*number of parents who have clones on lower levels in the pedigree*/
  int *parents;        /*indexes of parents*/
  int *children;       /*indexes of children*/
  int *pind;           /*indexes of parents in the COMP structure*/
  int *cind;           /*indexes of children in the COMP structure*/

  int *knowndata;      /*indexes (in component) of parents, whose genetic data is known in advance*/
  pair *cloned;        /*cloned[i][0]=the index (in component) of a parent, who has clones at lower levels
			 cloned[i][1]=the highest level on which the parent has clones*/
};
typedef struct connected connected_component;


struct components_in_generation
{
  int n;                           /*number of individuals (=children) at level*/
  int ncomponents;                 /*number of components*/
  int nparents;                    /*number of parents*/
  int *individuals;                /*list of indexes of individuals (=children)*/
  int *parents;                    /*list of indexes of parents*/  
  int *c_comp;                     /*indexes of children's components (child=0...(level's_size-1))*/
  int *p_comp;                     /*indexes of parents' components (parent=0...(nparents-1))*/
  int *pind_in_comp;               /*indexes of parents 0...(nparents-1) in their component */
  int *cind_in_comp;               /*indexes of children 0...(level's size-1) in their component*/ 
  connected_component *components; /*list of components*/
  int *order;                      /*ordering of the components for calculation of kinship method scores*/
  pair *parent_at_level;           /*for each individual gives the highest level on which he/she is a parent 
				     and the index as a parent at that level. 
				     value is (-1,-1) if individual has no children*/
};
typedef struct components_in_generation COMP;


/**************** Data strutures *****************************************************************/


struct data_in_component
{
  pair **(tensor);    /*genetic data, has nconfig[comp] rows, 
			and COMP[generation].components[comp].nfathers + ...nmothers columns.*/
  double *proba;
};
typedef struct data_in_component c_list; 

struct configuration
{
  int *component;     /*list of componentwise configurations */
};
typedef struct configuration config;

struct data_in_generation
{  
  int prev_config;     /* index of the current configuration in the previous generation's config list*/
  int *nconfig;        /* componentwise number of c_lists*/
  c_list *comp;        /* list of componentwise c_lists, has COMP[generation].ncomponents members */  
  config *ordered;     /* list of componentwise configurations ordered according to scores (n elements)*/
  long n;              /* number of product configurations that are ordered*/
  double log_n;        /* log of n */
}; 
typedef struct data_in_generation g_list; 




/***************************** FUNCTIONS *****************************/



/*** APE_initialize.c ***/

void build_pedigree(pair** genes, g_list *G, COMP *C, int marker);
void check_consistency_of_component(connected_component *C, int g, int c,int **fixed_parents, pair** genes, int marker, double ***conditionalF);
void check_consistency_of_components(COMP *C,int N_levels,int **fixed_parents, pair** genes, int marker, double ***conditionalF);
void check_initial_pedigree(int **parents, int n);
double check_kinship_arrays(int ind0,int ind1,PARENTAL *parental,double ***kinship,double ***diff,int g,int diffg);
double *count_known_data(pair **genes, int marker, COMP *C);
double ***count_kinship(COMP *C, int **fixed_parents);
double ***count_kinship2(COMP *C, int **fixed_parents,PARENTAL *parental);
void count_level(int i,int **fixed_parents,int *levels);
pair **find_grandp_origins(pair **genes, int **fixed_parents);
PARENTAL* find_parental_info(COMP *C);
void infer_from_homozygosity(int **fixed_parents,pair **genes);
CHILDREN *list_children(int **fixed_parents);
void make_components(COMP *C, int **fixed_parents, int* knowndata, int *levels);
double **readAfreq(FILE *fp);
int *readData(FILE *datafile, pair **genes, int *N_knowndata);
void read_parameters(FILE *args, char* p, char *d, char *a,int *N_markers, int *N_c, int *N_o, double *kappa, double *diff_exp, int *max, int *allowed_jumps);
int **readPedigree(FILE *pedfile, int *lastindex);
int *read_sexes(FILE *fp);
double single_kinship_with_arrays(int i,int j,int **parents,PARENTAL *parental,double ***kinship,double ***diff,int g,int diffg);
void total_configuration(pair **genes, pair **origins,int **fixed_parents, COMP *C, double **afreq,int nonancestral);
double*** transmission_probabilities( pair **genes, int **fixed_parents, COMP *C,int l, int n_alleles, double *afreq);



/*** APE_general.c ***/

void alloc_error(char *c);
int compatible(int a, int parent[2]);
double conditional_probability(int i,pair genes,double ***conditionalF,int n_alleles);
int contains(int a, pair gene);
void delete_combination(pair **ttensor, int *i, int *ntensor, int *ntensor0);
void delete_combination2(pair **ttensor,int*ik, int *ntensor);
int find_configurations(g_list *G, COMP *C, int g, int **fixed_parents, pair** genes, int marker, double ***conditionalF);
void free_glist(g_list *glist, int ncomponents);
void join_identical2(pair **ttensor,int *i,int *ntensor,int nparents);
int ordinal(int i, int *array, int n);
void sigproc();
int subtype(pair GT1, pair GT2);
pair** update_list(int child[2],int f,int F,int m,int M,pair **ttensor,int *i,int *ntensor,int *ntensor0, double ***conditionalF,int nparents,int n_alleles);
void update_tgenes(pair *tgenes,int g, COMP *C,g_list *G,int step);


/*** APE_order.c ***/

int compar2(const void *A, const void *B);
int compare_group(const void *A, const void *B);
int count_different_genes(pair genes, pair genes2);
double diff_component(pair *tgenes, COMP *C, int g,int c, double ***kinship);
double diff_group_to_component(pair *tgenes, COMP *C, int g,int ngroup, double ***kinship);
void find_component_ordering(COMP *C, double ***kinship);
void order_component(pair **ttensor,double *proba, int ntensor);
void order_generation(g_list *G, COMP *C, int g, double ***kinship, pair* tgenes, double *ratio_known);


/*** APE_print.c ***/

int check_compatibility(FILE *data,pair **genes);
int check_consistency(pair **genes, int **fixed_parents);
void draw_pedigree(char *pedfile);
void print_defined_genes(pair **genes,COMP *C);
void print_inbred(FILE *fp, double **kinship);
void print_combinations(config *ordered, int n, int ncomponents, int g,FILE *fp);
void print_comments(int success);
void print_components(COMP *C, FILE *fp);
void print_components_info(COMP *C, FILE *fp);
void print_connected_components(int **parents);
void print_conditionalF(double ***conditionalF, COMP *C,FILE *fp,int n_alleles);
void print_generation(g_list *G,COMP *C, int g,FILE *fp);
void print_genotypes(g_list *G, COMP *C, int g, FILE *fp);
void print_inconsistencies(FILE *out,UL **inconsistencies,COMP *C,UL no_cons,double frac);
void print_kinship(FILE *fp,COMP *C,double ***kinship);
void print_kinship_matrix(double **kinship,FILE *fp);
void print_locus(char *pedfile, pair **genes,int marker,FILE *out);
void print_offspring(CHILDREN *offspring,FILE *out);
void print_parental_info(FILE *out, PARENTAL* parental);
void print_pedigree(char *pedfile, pair **genes, int last_marker, FILE *fp);
void print_pedigree_info(FILE *fp,COMP *C,int *sexes);
void print_phase(int **phase,FILE *fout);
void print_tgenes(pair *tgenes, FILE *fp);
void print_total(char *pedfile,pair **genes,pair **origins,int *levels,FILE *fout);
int set_component(int ind,int color,int **parents,int *comp);


/* Global variables (many of these are defined again in the parameter file) */

unsigned long no_cons=0; //number of inconsistencies encountered
int N_markers=1;
int *N_alleles;  /*shall contain N_markers many positive entries*/
int M_comp=0;
int M_lev=10;
int M_clu=1000;
double kappa=1.0;
double diff_exp=1.0; //exponent for the difference in the variation coeffifient calculation
double inconsistency_frac=0.1; //How large a fraction of inconsistencies is required to output the component.
int N_levels;
int N_individuals;
int N_knowndata;
FILE *times; 
double sek=0.0;
clock_t kello;
int n_infer[2]={0,0};





