/* APE (Allelic Path Explorer) v.1.0
   searches for a consistent genotypic configuration 
   when the known pedigree structure 
   and genetic data for some individuals are given. 


   Copyright (C) Matti Pirinen
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   version 2 as published by the Free Software Foundation.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program (gpl.txt); if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
   MA  02110-1301, USA.

   www.gnu.org/licenses/gpl.html
*/

/* FILE: APE_initialize.c
   Contains functions for initializing data structures of APE.
   See APE.h for list of functions.
*/



void build_pedigree(pair** genes, g_list *G, COMP *C, int marker)
{

  int g,comp,i,a1,a2,ind,consistent;
  int *configuration;

  if( (configuration = (int *) malloc(C[N_levels-2].ncomponents*sizeof(int))) == NULL)
    alloc_error("build_pedigree with configuration");

  for(i=0;i<C[N_levels-2].ncomponents;i++)
    configuration[i]=0;

  for(g=N_levels-2;g>=0;g--)
    {
      for(comp=0;comp < C[g].ncomponents; comp++)
	{
	  for(i=0;i < C[g].components[comp].nparents; i++)
	    {
	      ind=C[g].components[comp].parents[i];
	      a1=G[g].comp[comp].tensor[configuration[comp]][i][0];
	      a2=G[g].comp[comp].tensor[configuration[comp]][i][1];
	      consistent=compatible(a1, genes[ind][marker]);
	      if(!consistent) 
		{
		  printf("\nERROR Not consistent pedigree at level %d.\n",g);
		  exit(5);
		}
	      consistent=compatible(a2, genes[ind][marker]);
	      if(!consistent) 
		{
		  printf("\nERROR Not consistent pedigree at level %d.\n",g);
		  exit(5);
		}
	    }
	}

      if(g==N_levels-2)
	free(configuration);
      if(g>0)
	configuration=G[g-1].ordered[G[g].prev_config].component;
    }


}


void check_consistency_of_components(COMP *C,int N_levels,int **fixed_parents, pair** genes, int marker, double ***conditionalF)
{
  int c,l;
  FILE *out;

  for(l=0;l<N_levels;l++)
    for(c=0;c<C[l].ncomponents;c++)
      check_consistency_of_component(C[l].components+c,l,c,fixed_parents,genes,marker,conditionalF);

  out=fopen(INFERRED_OUTPUT,"a");
  fprintf(out,"\ninferred %d cases from homozygosity and some ",n_infer[0]);
  fprintf(out,"alleles on %d components from offspring's data\n",n_infer[1]);
  fclose(out);
}


void check_consistency_of_component(connected_component *C, int g, int c,int **fixed_parents, pair** genes, int marker, double ***conditionalF)
{

  /*
    Checks whether genes are consistent with pedigree in component 
    C (i.e. component c at level g).
    If exactly one configuration is possible, then sets genes accordingly.
  */

  int t,i,j;
  int nchildren,Father,Mother,ntensor,ntensor0,nparents,child,father,mother;
  pair *ctensor,**(ttensor);

  nchildren=C->nchildren;
  nparents = C->nparents;

  if( (ctensor = (pair*) malloc(nchildren*sizeof(pair))) == NULL)
    alloc_error("find_configurations with 'ctensor'");
  
  for(i=0; i < nchildren; i++)
    {
      ctensor[i][0]=genes[C->children[i]][marker][0];
      ctensor[i][1]=genes[C->children[i]][marker][1];
    }

  if( (ttensor = (pair**) malloc( sizeof(pair*))) == NULL)
    alloc_error("find_configurations with 'ttensor'");

  if( (ttensor[0] = (pair*) calloc(nparents, sizeof(pair))) == NULL)
    alloc_error("find_configurations with 'ttensor'");
  
  ntensor=1;
  
  for(j=0; j<nparents; j++)
    {
      t=C->parents[j];
      
      ttensor[0][j][0]=genes[t][marker][0];
      ttensor[0][j][1]=genes[t][marker][1];
    }

  for(j=0; j<nchildren; j++)
    {
      /* Proposal: One doesn't have to check cases when child is 0/0.*/
      child=C->children[j];

      Father = fixed_parents[child][0];
      father = ordinal(Father,C->parents,nparents);
      
      Mother = fixed_parents[child][1];
      mother = ordinal(Mother,C->parents,nparents);

      ntensor0=ntensor;

      for(i=0;i<ntensor0;i++)
	ttensor=update_list(ctensor[j],father,Father,mother,Mother,ttensor,&i,&ntensor,&ntensor0, conditionalF,nparents,N_alleles[marker]);

      for(i=0;i<ntensor;i++)
	join_identical2(ttensor,&i,&ntensor,nparents);
      
    }
    
  if(ntensor==0)
    {   /* no consistent configurations found.*/
      
      free(ttensor);
      free(ctensor);
      
      fprintf(stdout,"Inconsistent data at marker %d, component %d, level %d.\nParents:\n",marker+1,c,g);
      for(i=0;i<nparents;i++)
	fprintf(stdout,"%d: (%d,%d)\n",C->parents[i]+1,genes[C->parents[i]][marker][0],genes[C->parents[i]][marker][1]);
      fprintf(stdout,"Children:\n");
      for(i=0;i<nchildren;i++)
	{
	  child=C->children[i];
	  fprintf(stdout,"%d: (%d,%d)",child+1,genes[child][marker][0],genes[child][marker][1]);
	  fprintf(stdout," ,father: %d, mother: %d\n",fixed_parents[child][0]+1,fixed_parents[child][1]+1);
	}
      printf("\nNote that inconsistency may be also due to conditional probabilities.\n");
      printf("In that case, check the file conditional_probabilities to see parents' possible genotypes.\n"); 
      exit(3);
    }
  if(ntensor==1)
    {   /* exactly one possible configuration found*/
      t=0;
      for(i=0;i<nparents;i++)
	{
	  if( (ttensor[0][i][0]!=genes[C->parents[i]][marker][0]) || (ttensor[0][i][1]!=genes[C->parents[i]][marker][1]))
	    t=1;
	  genes[C->parents[i]][marker][0]=ttensor[0][i][0];
	  genes[C->parents[i]][marker][1]=ttensor[0][i][1];

	}
      if(t)
	{
	  FILE* infer=fopen(INFERRED_OUTPUT,"a");
	  fprintf(infer,"Exactly one configuration possible for marker %d on component %d at level %d:\n",marker+1,c,g);
	  for(i=0;i<nparents;i++)
	    fprintf(infer,"%d: (%d,%d)\n",C->parents[i]+1,ttensor[0][i][0],ttensor[0][i][1]);
	  fprintf(infer,"\n");
	  fclose(infer);
	  n_infer[1]++;
	}
    }

  for(i=0;i<ntensor;i++)
    free(ttensor[i]);

  free(ttensor);
  free(ctensor);
  
}




void check_initial_pedigree(int **parents, int n)
{

  /*Checks that no individual is a mother and a father 
    and that an individual can not have exactly one 
    defined parent in the pedigree.
  */

  int i;
  int * sex;

  if( (sex = (int*) calloc(n,sizeof(int))) == NULL )
    alloc_error("check_initial_pedigree\n");

  for(i=0;i<n;i++)
    {
      if(parents[i][0]>-1 && parents[i][1]>-1){
	if(sex[parents[i][0]] == 2) 
	  {
	    fprintf(stderr,"\nError: pedfile assigns individual %d to be both a mother and a father!\n\n",parents[i][0]+1);
	    exit(3);
	  }
	if(sex[parents[i][1]]==1)
	  {
	    fprintf(stderr,"\nError: pedfile assigns individual %d to be both a mother and a father!\n\n",parents[i][1]+1);
	    exit(3);
	  }
	sex[parents[i][0]]=1;
	sex[parents[i][1]]=2;
      }
      else
	{
	  if(parents[i][0]>-1 || parents[i][1]>-1)
	    {
	      fprintf(stderr,"\nError: pedfile assigns only one parent to the individual %d!\n\n",i+1);
	      exit(3);
	    }
	}
    }
  
}

double check_kinship_arrays(int ind0,int ind1,PARENTAL *parental,double ***kinship,double ***diff,int g,int diffg)
{
  /*
    Checks whether the coefficient for ind0 and ind1 has already been 
    calculated and stored in the kinship or diff arrays above level g.
    If diffg==1, then the array diff[g][][] is also available.
  */

  int l0,l1,lev0,lev1;

  for(l0=parental[ind0].nlevels-1;l0>=0;l0--)
    {
      if((lev0=parental[ind0].levels[l0])<g || (!diffg && lev0==g))
	break;

      for(l1=parental[ind1].nlevels-1;l1>=0;l1--)
	{
	  if((lev1=parental[ind1].levels[l1])<g || (!diffg && lev1==g))
	    break;

	  if(lev0==lev1 && lev0>g)
	    return kinship[lev0][parental[ind0].index[l0]][parental[ind1].index[l1]];

	  if(lev0-lev1==1)
	    return diff[lev1][parental[ind1].index[l1]][parental[ind0].index[l0]];

	  if(lev1-lev0==1)
	    return diff[lev0][parental[ind0].index[l0]][parental[ind1].index[l1]];
	}

    }

  return -1.0; //not found
}


double *count_known_data(pair **genes, int marker, COMP *C)
{
  /*Returns an array that contains the ratio of genes 
    that are defined for the parents of each level.*/

  int i,j,ind;
  double *known;

  if((known=(double*) calloc(N_levels,sizeof(double)))==NULL)
     alloc_error("count_known_data\n");
  

  for(i=0;i<N_levels-1;i++)
    {
      for(j=0;j<C[i].nparents;j++)
	{
	  ind=C[i].parents[j];
	  if(genes[ind][marker][0]>0)
	    known[i] += 1.0;
	  if(genes[ind][marker][1]>0)
	    known[i] += 1.0;
	}
      known[i] /= 2.0*C[i].nparents;
      //printf("Level %d, ratio %lf\n",i,known[i]);
    }

  return known;
}



double ***count_kinship(COMP *C, int **fixed_parents)
{

  /*
    Returns the array that contains pairwise kinship coefficients of parents
    of each level 0,...,N_levels-2.
    First all pairwise coefficients are calculated. 
    See count_kinship2 for a faster algorithm.
    (This is used to check that the faster one gives correct results.
    And can also be used with some other functions like print_inbred().)
    See 
    Lange: Mathematical and Statistical Methods for Genetic Analysis 2.ed 2002, 
    for the algorithm.
  */

  int g,i,ind,ind2,f,m,j,l;

  double **kinship;
  double ***final; 

  if( (kinship = (double **) malloc(N_individuals*sizeof(double*)))==NULL)
    alloc_error("count_kinship\n");
  
  for(i=0; i<N_individuals; i++)
    if( (kinship[i] = (double *) malloc(N_individuals*sizeof(double)))==NULL)
      alloc_error("count_kinship\n");


  g=N_levels-1;
  for(i=0;i<C[g].n;i++)
    {
      ind=C[g].individuals[i];
      kinship[ind][ind]=0.5;

      for(j=0;j<i;j++)
	{
	  ind2=C[g].individuals[j];
	  kinship[ind][ind2]=kinship[ind2][ind]=0.0;
	}
    }


  for(g=N_levels-2;g>=0;g--)
    {
      for(i=0;i<C[g].n;i++)
	{
	  ind=C[g].individuals[i];
	  f=fixed_parents[ind][0];
	  m=fixed_parents[ind][1];
	  kinship[ind][ind]=0.5+0.5*kinship[f][m];
	  
	  for(j=0;j<i;j++)
	    {
	      ind2=C[g].individuals[j];
	      kinship[ind][ind2]=kinship[ind2][ind]=0.5*kinship[ind2][f]+0.5*kinship[ind2][m];
	    }

	  for(l=N_levels-1;l>g;l--)
	    {
	      for(j=0;j<C[l].n;j++)
		{
		  ind2=C[l].individuals[j];
		  kinship[ind][ind2]=kinship[ind2][ind]=0.5*kinship[ind2][f]+0.5*kinship[ind2][m];
		}
	    }
	}
    }


  if( (final = (double ***) malloc((N_levels-1)*sizeof(double**)))==NULL)
    alloc_error("count_kinship\n");

  for(g=0;g<N_levels-1;g++)
    {
      
      if( (final[g] = (double **) malloc(C[g].nparents*sizeof(double*)))==NULL)
	alloc_error("count_kinship\n");
  
      for(i=0; i<C[g].nparents; i++)
	{
	  if( (final[g][i] = (double *) malloc(C[g].nparents*sizeof(double)))==NULL)
	    alloc_error("count_kinship\n");

	  for(j=0;j<C[g].nparents;j++)
	    final[g][i][j]=kinship[C[g].parents[i]][C[g].parents[j]];
	}
    }

  for(i=0;i<N_individuals;i++)
    free(kinship[i]);
  free(kinship);
      
  return final;

}



double ***count_kinship2(COMP *C, int **fixed_parents,PARENTAL *parental)
{

  /*
    Returns the array that contains pairwise kinship coefficients for the parents 
    of each level.
    See 
    Lange: Mathematical and Statistical Methods for Genetic Analysis 2.ed 2002, 
    for the algorithm.
  */
  

  int g,i,ind0,ind1,j;

  double ***kinship;
  double ***diff;

  if( (kinship = (double ***) malloc((N_levels-1)*sizeof(double**)))==NULL)
    alloc_error("count_kinship\n");

  if( (diff = (double ***) malloc((N_levels-2)*sizeof(double**)))==NULL)
    alloc_error("count_kinship\n");
  
  for(g=0;g<N_levels-1;g++)
    {      
      if( (kinship[g] = (double **) malloc(C[g].nparents*sizeof(double*)))==NULL)
	alloc_error("count_kinship\n");
  
      if(g<N_levels-2)
	if( (diff[g] = (double **) malloc(C[g].nparents*sizeof(double*)))==NULL)
	  alloc_error("count_kinship\n");

      for(i=0; i<C[g].nparents; i++)
	{
	  if( (kinship[g][i] = (double *) malloc(C[g].nparents*sizeof(double)))==NULL)
	    alloc_error("count_kinship\n");

	  if(g<N_levels-2)
	    if( (diff[g][i] = (double *) malloc(C[g+1].nparents*sizeof(double)))==NULL)
	      alloc_error("count_kinship\n");
	}
    }

  if(N_levels>1)
    g=N_levels-2;
  else
    g=0;

  for(i=0;i<C[g].nparents;i++)
    {
      kinship[g][i][i]=0.5;

      for(j=0;j<i;j++)
	kinship[g][i][j]=kinship[g][j][i]=0.0;
    }

  for(g=N_levels-3;g>=0;g--)
    {

      //kinship between parents[g] and parents[g+1] will be stored in diff[g][][]
      for(i=0;i<C[g].nparents;i++)
	{
	  ind0=C[g].parents[i];
	  
	  for(j=0;j<C[g+1].nparents;j++)
	    {
	      ind1=C[g+1].parents[j];
	      diff[g][i][j]=single_kinship_with_arrays(ind0,ind1,fixed_parents,parental,kinship,diff,g,0);
	    }
	}

      //diff[g][][] is now in use
      for(i=0;i<C[g].nparents;i++)
	{
	  ind0=C[g].parents[i];

	  if(fixed_parents[ind0][0]<0)
	    kinship[g][i][i]=0.5;
	  else
	    kinship[g][i][i]=single_kinship_with_arrays(ind0,ind0,fixed_parents,parental,kinship,diff,g,1);
	    
	  //finding kinship[g][i][j], j<i
	  for(j=0;j<i;j++)
	    {
	      ind1=C[g].parents[j];
	      kinship[g][i][j]=single_kinship_with_arrays(ind0,ind1,fixed_parents,parental,kinship,diff,g,1);
	      kinship[g][j][i]=kinship[g][i][j];
	    }

	}
    }

  for(g=0;g<N_levels-2;g++)
    {
      for(i=0;i<C[g].nparents;i++)
	free(diff[g][i]);
      free(diff[g]);
    }
  free(diff);

  return kinship;

}



void count_level(int i,int **fixed_parents,int *levels)
{

  int f=0,m=0;

  if(levels[i]==-1)
    {

      if(fixed_parents[i][0]==-1 && fixed_parents[i][1]==-1)
	levels[i]=0;
      else
	{
	  if(fixed_parents[i][0]>-1)
	    {
	      count_level(fixed_parents[i][0],fixed_parents,levels);
	      f=levels[fixed_parents[i][0]];
	    }
	  if(fixed_parents[i][1]>-1)
	    {
	      count_level(fixed_parents[i][1],fixed_parents,levels);
	      m=levels[fixed_parents[i][1]];
	    }
	  if(f<m)
	    levels[i]=m+1;
	  else
	    levels[i]=f+1;
	}
    }
}




pair **find_grandp_origins(pair **genes, int **fixed_parents)
{

  /*Returns some grandparental origins compatible with the genes.
    No recombinations are considered here.
    As a side effect assigns phases for the array 'genes' so that 
    genes[][][0] is inherited from the father and genes[][][1] from the mother.
  */

  int i,f,m,a,b,marker,p;
  pair **origins;
  pair parents;

  if( (origins = (pair **) malloc(N_individuals * sizeof(pair *))) == NULL )
    alloc_error("find_grandp_origins\n");

  for(i=0;i<N_individuals;i++)
    {
      if( (origins[i] = (pair *) malloc(N_markers * sizeof(pair))) == NULL )
	alloc_error("find_grandp_origins\n");
    }

  /* finding phases and arranging genes so that 
     the first is from the father and the second from the mother.*/ 

  for(i=0; i<N_individuals; i++)
    {

      f=fixed_parents[i][0];
      m=fixed_parents[i][1];

      if(f > -1 && m > -1)
	{

	  for(marker=0;marker<N_markers; marker++)
	    {

	      a=genes[i][marker][0];
	      b=genes[i][marker][1];

	      if( contains(a,genes[f][marker]) && !contains(a,genes[m][marker]) )
		continue;

	      if( contains(a,genes[m][marker]) && !contains(a,genes[f][marker]) )
		{
		  genes[i][marker][0]=b;
		  genes[i][marker][1]=a;
		  continue;
		}

	      if( contains(b,genes[f][marker]) && !contains(b,genes[m][marker]) )
		{
		  genes[i][marker][0]=b;
		  genes[i][marker][1]=a;
		  continue;
		}

	      if( contains(b,genes[m][marker]) && !contains(b,genes[f][marker]) )
		continue;

	      /*Here either a == 0 == b 
	      or one of a and b is 0 and the other is held by both of the parents 
	      or a != 0 != b and both parents have the genotype {a,b}*/

	      if(rand()%2==1)
		{
		  genes[i][marker][0]=b;
		  genes[i][marker][1]=a;
		  continue;
		}
	    }

	}
      else // for founders
	for(marker=0;marker<N_markers; marker++)
	  if(rand()%2==1)
	    {
	      a=genes[i][marker][0];
	      genes[i][marker][0]=genes[i][marker][1];
	      genes[i][marker][1]=a;
	    }
    }

  /*genotypes arranged according to parental phases*/

  /*finding grandparental origins of the genes*/

  for(i=0; i<N_individuals; i++)
    {

      parents[0]=fixed_parents[i][0];
      parents[1]=fixed_parents[i][1];

      if(parents[0] > -1 && parents[1] > -1)
	{
	  for(marker=0;marker<N_markers;marker++)
	    {
	      
	      for(p=0;p<2;p++)
		{
		  if(genes[i][marker][p] == 0)
		    {
		      origins[i][marker][p] = rand()%2;
		      continue;
		    }

		  if(genes[parents[p]][marker][0] == genes[parents[p]][marker][1])
		    {
		      origins[i][marker][p] = rand()%2;
		      continue;
		    }

		  if(genes[i][marker][p] == genes[parents[p]][marker][0])
		    {
		      origins[i][marker][p] = 1;
		      continue;
		    }

		  if(genes[i][marker][p] == genes[parents[p]][marker][1])
		    {
		      origins[i][marker][p] = 0;
		      continue;
		    }

		  printf("ERROR in grandparental origins.\n");
		  exit(5);
		}
	    }
	}
      else /*for founders*/
	for(marker=0;marker<N_markers;marker++)
	  for(p=0;p<2;p++)
	    origins[i][marker][p] = rand()%2;
    }

  return origins;

}

PARENTAL* find_parental_info(COMP *C)
{
  int i,g,ind;
  PARENTAL *parental;

  if((parental=(PARENTAL*) malloc(N_individuals*sizeof(PARENTAL)))==NULL)
    alloc_error("find_parental_info");

  for(i=0;i<N_individuals;i++)
    {
      if((parental[i].levels=(int*) malloc(sizeof(int)*(N_levels-1)))==NULL)
	alloc_error("find_parental_info");
      if((parental[i].index=(int*) malloc(sizeof(int)*(N_levels-1)))==NULL)
	alloc_error("find_parental_info");

      parental[i].nlevels=0;
    }

  for(g=0;g<N_levels-1;g++)
    {
      for(i=0;i<C[g].nparents;i++)
	{
	  ind=C[g].parents[i];
	  parental[ind].levels[parental[ind].nlevels]=g;
	  parental[ind].index[parental[ind].nlevels++]=i;
	}
    }


  for(i=0;i<N_individuals;i++)
    {
      if(parental[i].nlevels)
	{
	  if((parental[i].levels=(int*) realloc(parental[i].levels,parental[i].nlevels*sizeof(int)))==NULL)
	    alloc_error("find_parental_info");
	  if((parental[i].index=(int*) realloc(parental[i].index,parental[i].nlevels*sizeof(int)))==NULL)
	    alloc_error("find_parental_info");
	}
    }
  
  return parental;

}


void infer_from_homozygosity(int **fixed_parents,pair **genes)
{
  int i,marker,a,f,m,count;
  FILE *infer=fopen(INFERRED_OUTPUT,"w");

  fprintf(infer,"Inferring genotypes from known data...\n");
  for(i=0;i<N_individuals;i++)
    {
      f=fixed_parents[i][0];
      m=fixed_parents[i][1];

      if(f>-1 && m>-1)
	for(marker=0;marker<N_markers;marker++)
	  {
	    a=genes[i][marker][0];
	    if(a!=0 && a==genes[i][marker][1]) /*Child is a nontrivial homozygote*/
	      {
		count=compatible(a,genes[f][marker])*compatible(a,genes[m][marker]);
		if(!count)
		  {
		    fprintf(stdout,"ERROR (in infer_from_homozygosity) at marker %d\nindividual %d (%d,%d)\n",marker+1,i+1,genes[i][marker][0], 
			    genes[i][marker][1]);
		    fprintf(stdout,"father %d (%d,%d)\n",f+1,genes[f][marker][0], 
			    genes[f][marker][1]);
		    fprintf(stdout,"mother %d (%d,%d)\n",m+1,genes[m][marker][0], 
			    genes[m][marker][1]);
		    exit(3);
		  }
		if(count>1)
		  {
		    n_infer[0]+=count/2;
		    fprintf(infer,"Inferred from homozygote child %d (%d,%d):\n",i+1,a,a);
		    fprintf(infer,"Father %d (%d,%d), Mother %d (%d,%d).\n",f+1,genes[f][marker][0], 
			    genes[f][marker][1],m+1,genes[m][marker][0],genes[m][marker][1]);
		  }
	      }
	    else
	      {
		a=genes[f][marker][0];
		if(a!=0 && a==genes[f][marker][1]) /*Father is a nontrivial homozygote*/
		  {
		    count=compatible(a,genes[i][marker]);
		    if(!count)
		      {
			fprintf(stdout,"ERROR (in infer_from_homozygosity) at marker %d\nindividual %d (%d,%d)\n",marker+1,i+1,genes[i][marker][0], 
				genes[i][marker][1]);
			fprintf(stdout,"father %d (%d,%d)\n",f+1,genes[f][marker][0], 
				genes[f][marker][1]);
			exit(3);
		      }
		    if(count==2)
		      {
			n_infer[0]++;
			fprintf(infer,"Inferred from homozygote father %d (%d,%d):\n",f+1,a,a);
			fprintf(infer,"Child %d (%d,%d).\n",i+1,genes[i][marker][0], 
				genes[i][marker][1]);
		      }
		  }
	      
		a=genes[m][marker][0];
		if(a!=0 && a==genes[m][marker][1]) /*Mother is a nontrivial homozygote*/
		  {
		    count=compatible(a,genes[i][marker]);
		    if(!count)
		      {
			fprintf(stdout,"ERROR (in infer_from_homozygosity) at marker %d\nindividual %d (%d,%d)\n",marker+1,i+1,genes[i][marker][0], 
				genes[i][marker][1]);
			fprintf(stdout,"mother %d (%d,%d)\n",m+1,genes[m][marker][0], 
				genes[m][marker][1]);
			exit(3);
		      }
		    if(count==2)
		      {
			n_infer[0]++;
			fprintf(infer,"Inferred from homozygote mother %d (%d,%d):\n",m+1,a,a);
			fprintf(infer,"Child %d (%d,%d).\n",i+1,genes[i][marker][0], 
				genes[i][marker][1]);
		      }
		  }
	      }
	  }
      
    }
  fclose(infer);
}

CHILDREN *list_children(int **fixed_parents)
{
  /* Returns the CHILDREN structure for the given pedigree.*/

  int i;
  CHILDREN *c;

  if( (c=(CHILDREN *) malloc(N_individuals*sizeof(CHILDREN)) )== NULL )
    alloc_error("list_children");

  for(i=0;i<N_individuals;i++)
    c[i].nchildren=0;

  for(i=0;i<N_individuals;i++)
    {
      if((fixed_parents[i][0] > -1) && (fixed_parents[i][1] > -1))
	{
	  c[fixed_parents[i][0]].nchildren++;
	  c[fixed_parents[i][1]].nchildren++;
	}
    }

  for(i=0;i<N_individuals;i++)
    {
      if((c[i].children = (int *) malloc(c[i].nchildren*sizeof(int))) == NULL)
	alloc_error("list_children with c.children");
      c[i].nchildren=0;
    }

  for(i=0;i<N_individuals;i++)
    {
      if((fixed_parents[i][0] > -1) && (fixed_parents[i][1] > -1))
	{
	  c[fixed_parents[i][0]].children[c[fixed_parents[i][0]].nchildren++]=i;
	  c[fixed_parents[i][1]].children[c[fixed_parents[i][1]].nchildren++]=i;
	}
    }

  return c;
}



void make_components(COMP *C, int **fixed_parents, int* knowndata, int *levels)
{

  /*Fills the Component structure C*/

  int g,i,j,f,m,c,new;
  int nchildren,ncomponents,nparents=0;
  int *child, *parentc, *com_size[2],*parents;
  int *clones_at_level;
  pair *parent_at_level;



  if( (clones_at_level = (int*) malloc(N_individuals*sizeof(int))) == NULL)
    alloc_error("make_components with clones_at_level");

  if( (parent_at_level = (pair*) malloc(N_individuals*sizeof(pair))) == NULL)
    alloc_error("make_components with parent_at_level");

  for(i=0;i<N_individuals;i++)
    {
      levels[i]=N_levels-1-levels[i];
      clones_at_level[i]=-1;
      parent_at_level[i][0]=-1;
      parent_at_level[i][1]=-1;
    }

  for(g=0;g<N_levels;g++)
    C[g].n=0;

  for(i=0;i<N_individuals;i++)
   C[levels[i]].n++;
  
  for(g=0;g<N_levels;g++)
    {
      if( (C[g].individuals = (int*) malloc(C[g].n*sizeof(int))) == NULL)
	alloc_error("make_components with C.individuals");
      C[g].n=0;
    }
  
  for(i=0;i<N_individuals;i++)
    C[levels[i]].individuals[C[levels[i]].n++]=i;



  for(g=0; g < N_levels-1; g++)
    {
      nchildren = C[g].n;

      if( (child = (int *) calloc(nchildren,sizeof(int))) == NULL) 
	alloc_error("make_components with child");

      if( (parents = (int *) malloc(sizeof(int))) == NULL )
	alloc_error("make_components with parents");
     

      if( (com_size[0] = (int *) malloc(nchildren*sizeof(int))) == NULL) 
	alloc_error("make_components with com_size");
      if( (com_size[1] = (int *) malloc(nchildren*sizeof(int))) == NULL) 
	alloc_error("make_components with com_size");

      ncomponents=0;
      nparents=0;

      for(i = 0; i< nchildren; i++)
	{
	  f=fixed_parents[C[g].individuals[i]][0];

	  new=(f>-1);
	  for(j=0; j<nparents && new; j++)
	    new=(f != parents[j]);

	  if(new)
	    {
	      nparents++;
	      if( (parents = (int*) realloc(parents,nparents*(sizeof(int)))) == NULL)
		alloc_error("make_components parents");
	      parents[nparents-1]=f;
	    }

	  m=fixed_parents[C[g].individuals[i]][1];

	  new=(m>-1);
	  for(j=0; j<nparents && new; j++)
	    new=(m != parents[j]);

	  if(new)
	    {
	      nparents++;
	      if( (parents = (int*) realloc(parents,nparents*(sizeof(int)))) == NULL)
		alloc_error("make_components with parents");
	      parents[nparents-1]=m;
	    }


	  if(f==-1)
	    child[i]=-1;

	}

      if( (parentc = (int *) malloc(nparents*sizeof(int))) == NULL) 
	alloc_error("make_components with parentc");

      for(i=0; i<nparents; i++)
	parentc[i]=-1;
     
      for(i=0; i<nchildren; i++)
	{

	  if(child[i]>-1)
	    {
	      c=C[g].individuals[i];
	      f=ordinal(fixed_parents[c][0],parents,nparents);
	      m=ordinal(fixed_parents[c][1],parents,nparents);
	      
	      if(parentc[f]<0)
		{
		  if(parentc[m]<0)
		    { /* initialize a new component */
		      com_size[0][ncomponents]=1;
		      com_size[1][ncomponents]=2;
		      child[i]=parentc[f]=parentc[m]=ncomponents++;
		    }
		  else
		    {/* add a new father and a child to an existing component */
		      child[i]=parentc[f]=parentc[m]; 
		      com_size[0][child[i]]++;
		      com_size[1][child[i]]++;
		    }
		}
	      else
		{
		  if(parentc[m]<0)
		    { /* add a new mother and a child to an existing component */
		      child[i]=parentc[m]=parentc[f];
		      com_size[0][child[i]]++;
		      com_size[1][child[i]]++;
		    }
		  else
		    { /* add a child to an existing component */
		      if(parentc[f]==parentc[m])
			{
			  child[i]=parentc[f];
			  com_size[0][child[i]]++;
			}
		      else
			{ /* unite two existing components */
			  int min_c = (parentc[f] < parentc[m]) ? parentc[f] : parentc[m];
			  int max_c = (parentc[f] > parentc[m]) ? parentc[f] : parentc[m];
			  child[i]=min_c;
			  com_size[0][min_c]+=com_size[0][max_c]+1;
			  com_size[1][min_c]+=com_size[1][max_c];
			  
			  for(j=0; j<i; j++)
			    {
			      if(child[j]==max_c) 
				child[j]=min_c;
			      else 
				if(child[j]>max_c) 
				  { 
				    com_size[0][child[j]--]--;
				    com_size[0][child[j]]++;
				  }
			    }
			  
			  for(j=0; j<nparents; j++)
			    {
			      if(parentc[j]==max_c) 
				parentc[j]=min_c;
			      else 
				{
				  if(parentc[j]>max_c) 
				    {
				      com_size[1][parentc[j]--]--;
				      com_size[1][parentc[j]]++;
				    }
				}
			    }
			  
			  ncomponents--;
			}
		      
		    }
		}
	  
	    }
	}
      
      
       /*Initializing COMP structure for the current generation g+1. */

      C[g].ncomponents=ncomponents;
      C[g].nparents=nparents;

      if((C[g].components = (connected_component *) malloc(ncomponents*sizeof(connected_component))) == NULL)
	alloc_error("make_components with C.components");

      if((C[g].pind_in_comp = (int *) malloc(nparents*sizeof(int))) == NULL)
	alloc_error("make_components with C.pind_in_comp"); 
      
      if((C[g].p_comp = (int *) malloc(nparents*sizeof(int))) == NULL)
	alloc_error("make_components with C.p_comp");

      if((C[g].cind_in_comp = (int *) malloc(nchildren*sizeof(int))) == NULL)
	alloc_error("make_components with C.cind_in_comp");

      if((C[g].c_comp = (int *) malloc(nchildren*sizeof(int))) == NULL)
	alloc_error("make_components with C.c_comp");

      if((C[g].parent_at_level = (pair *) malloc(nchildren*sizeof(pair))) == NULL)
	alloc_error("make_components with C.parent_at_level");

      if((C[g].parents = (int *) malloc(nparents*sizeof(int))) == NULL)
	alloc_error("make_components with C.parents");


      for(i=0;i<ncomponents;i++)
	{

	  C[g].components[i].nparents=0;
	  C[g].components[i].nchildren=0;
	  C[g].components[i].nknowndata=0;
	  C[g].components[i].ncloned=0;

	  if( (C[g].components[i].children = (int *) malloc( com_size[0][i]*sizeof(int) ) ) == NULL)
	    alloc_error("make_components with C.components.children");
	  if( (C[g].components[i].cind = (int *) malloc( com_size[0][i]*sizeof(int) ) ) == NULL)
	    alloc_error("make_components with C.components.children");
	  if( (C[g].components[i].parents = (int *) malloc( com_size[1][i]*sizeof(int) ) ) == NULL)
	    alloc_error("make_components with C.components.parents");
	  if( (C[g].components[i].pind = (int *) malloc( com_size[1][i]*sizeof(int) ) ) == NULL)
	    alloc_error("make_components with C.components.pind");
	  if((C[g].components[i].cloned = (pair *) malloc(sizeof(pair))) == NULL)
	    alloc_error("make_components with C.components.cloned");
	}

      for(i=0; i<nparents; i++)
	{
	  C[g].parents[i]=parents[i];
	  C[g].components[parentc[i]].parents[C[g].components[parentc[i]].nparents]=parents[i];
	  C[g].components[parentc[i]].pind[C[g].components[parentc[i]].nparents]=i;
	  C[g].p_comp[i]=parentc[i];
	  C[g].pind_in_comp[i] = C[g].components[parentc[i]].nparents++;
	  parent_at_level[parents[i]][0]=g;
	  parent_at_level[parents[i]][1]=i;

	  if(clones_at_level[parents[i]] > -1)
	    {
	      if( (C[g].components[parentc[i]].cloned = (pair*) realloc(C[g].components[parentc[i]].cloned,(++C[g].components[parentc[i]].ncloned)*sizeof(pair))) == NULL ) 
		alloc_error("make_components with C.components.cloned");
	      C[g].components[parentc[i]].cloned[C[g].components[parentc[i]].ncloned-1][0]=C[g].pind_in_comp[i];
	      C[g].components[parentc[i]].cloned[C[g].components[parentc[i]].ncloned-1][1]=clones_at_level[parents[i]];
	    }

	  if(levels[parents[i]] > g+1)
	    {
	      clones_at_level[parents[i]]=g;
	    }
	}

      for(i=0;i<ncomponents;i++)
	{
	  if(C[g].components[i].ncloned==0)
	    {
	      free(C[g].components[i].cloned);
	      C[g].components[i].cloned=NULL;
	    }
	}

      for(i=0; i<nchildren; i++)
	{
	  C[g].parent_at_level[i][0]=parent_at_level[C[g].individuals[i]][0];
	  C[g].parent_at_level[i][1]=parent_at_level[C[g].individuals[i]][1];
	  C[g].c_comp[i]=child[i];
	  if(child[i]>-1)
	    {
	      C[g].components[child[i]].children[C[g].components[child[i]].nchildren]=C[g].individuals[i];
	      C[g].components[child[i]].cind[C[g].components[child[i]].nchildren]=i;
	      C[g].cind_in_comp[i]=C[g].components[child[i]].nchildren++;
	    }
	}


      /* Setting indexes of individuals with known genotypes. */

      for(i=0; i<nparents; i++)
	{
	  for(j=0; j<N_knowndata; j++)
	    {
	      if(parents[i]==knowndata[j] )
		{
		  C[g].components[parentc[i]].nknowndata++;
		  if(C[g].components[parentc[i]].nknowndata==1)
		    {
		      if( (C[g].components[parentc[i]].knowndata = (int *) malloc(sizeof(int))) == NULL)
			alloc_error("make_components with C.components.knowndata");
		    }
		  else
		    {
		      if( (C[g].components[parentc[i]].knowndata = (int *) realloc(C[g].components[parentc[i]].knowndata, C[g].components[parentc[i]].nknowndata*sizeof(int)))==NULL)
			alloc_error("make_components with C.components.knowndata");
		    }
		  C[g].components[parentc[i]].knowndata[C[g].components[parentc[i]].nknowndata-1]=C[g].pind_in_comp[i];
		  break;
		}
	    }
	}

      free(child);
      free(parents);
      free(parentc);
      free(com_size[0]);
      free(com_size[1]);
    }

  C[N_levels-1].ncomponents=0;
  C[N_levels-1].components=NULL;
  C[N_levels-1].p_comp=NULL;
  C[N_levels-1].c_comp=NULL;
  C[N_levels-1].pind_in_comp=NULL;
  C[N_levels-1].cind_in_comp=NULL;
  free(clones_at_level);
  free(parent_at_level);
}



double **readAfreq(FILE *fp)
{

  int i,j;
  double **af;
  double sum;
  char c;

  if( (af = (double**) malloc(N_markers*sizeof(double *))) == NULL)
    alloc_error("readAfreq");

  for(i=0;i<N_markers;i++)
    if( (af[i] = (double *) calloc(N_alleles[i],sizeof(double))) == NULL)
      alloc_error("readAfreq");

  for(i=0;i<N_markers;i++)
    {
      sum=0.0;
      do
	{
	  c=fgetc(fp);
	}
      while(c=='\n' || c=='\t' || c==' ');
      if(c!='{')
	{
	  fprintf(stderr,"ERROR in afreq file with marker %d. Expected '{' for the first character.\n\n",i+1); 
	  exit(3);
	  }
      for(j=0;j<N_alleles[i];j++)
	{
	  fscanf(fp,"%lf",&af[i][j]);
	  if(af[i][j]<0.0 || af[i][j]>1.0)
	    {
	      fprintf(stderr,"ERROR in afreq file: allele %d of marker %d has invalid frequency %f.\n\n",j+1,i+1,af[i][j]);
	      exit(3);
	    }
	  sum += af[i][j];
	  c=fgetc(fp);
	  if(j<N_alleles[i]-1 && c!=',')
	    {
	      fprintf(stderr,"ERROR in afreq file with marker %d. Expected ',' right after the freq. of allele %d.\n\n",i+1,j+1); 
	      exit(3);
	    }
	  else if(j==N_alleles[i]-1 && c!='}')
	    {
	      fprintf(stderr,"ERROR in afreq file with marker %d. Expected '}' right after the freq. of allele %d.\n\n",i+1,j+1); 
	      exit(3);
	    }
	}
      if(fabs(sum-1.0)>0.01)
	{
	  fprintf(stderr,"ERROR in afreq file with marker %d. Sum is %f (not 1.0 +/- 0.01 as it should).\n\n",i+1,sum); 
	  exit(3);
	}
    }

  return af;

}


int *readData(FILE *datafile, pair **genes,int* N_knowndata)
{
 
  int i,ind,j,nonzero,k,l=0; 
  int *knowndata;

  fscanf(datafile,"%d\n",N_knowndata);

  if(*N_knowndata > N_individuals || *N_knowndata < 1)
    {
      fprintf(stderr,"ERROR: datafile has more individuals (%d) than the pedigree file (%d)!\n",*N_knowndata,N_individuals);
      exit(3);
    }

  if( (knowndata = (int *) malloc((*N_knowndata)*sizeof(int))) == NULL)
    alloc_error("readData\n");
  
  for (i=0,j=0;i<*N_knowndata && !feof(datafile);i++) 
    { 
      nonzero=0;
      fscanf(datafile,"%d\n ",&ind); 

      if(ind>N_individuals || ind < 1)
	{
	   fprintf(stderr,"ERROR: the index %d in the datafile is invalid!\n(Should be between 1 and %d\n",ind,N_individuals);
	   exit(3);
	}

      knowndata[j]=ind-1;

      for(k=0;k<2 && !feof(datafile);k++)
	{
	  for (l=0;l<N_markers && !feof(datafile);l++) 
	    {
	      fscanf( datafile, "%d  ",&genes[ind-1][l][k]);
	      if(genes[ind-1][l][k]<0 || genes[ind-1][l][k]>N_alleles[l])
		{
		  printf("ERROR in reading data.\nThe label of the allele at locus %d of individual %d is %d.\n",l+1,ind,genes[ind-1][l][k]);
		  printf("Accepted values are 0...%d.\n",N_alleles[l]);
		  exit(3);
		}
	      nonzero += (genes[ind-1][l][k]!=0);
	    }
	  fscanf(datafile,"\n");
	}
      
      if(l<N_markers)
	{
	  printf("ERROR in reading data.\nComplete data for %d individuals should be given in datafile.\n",*N_knowndata);
	  exit(3);
	}

      fscanf(datafile,"\n"); 
      if(nonzero) 
	++j;
    }

  if(i<*N_knowndata) // For loop has exited because of the end of file.
    {
      printf("ERROR in reading data.\nComplete data for %d individuals should be given in datafile.\n",*N_knowndata);
      exit(3);
    }

  if( (knowndata = (int *) realloc(knowndata,j*sizeof(int)))==NULL)
    alloc_error("readData\n");
  
  *N_knowndata=j;


  return knowndata;

}







void read_parameters(FILE *args, char *pedfile, char* datafile, char* afreqfile, int *N_markers,int *N_c, int *N_o, double *kappa, double *diff_exp,int *M_clu,int *allowed_jumps)
{

  int i;
  char label[40];

  fscanf(args,"%s\n",pedfile);
  fscanf(args,"%s\n",datafile);
  fscanf(args,"%s\n",afreqfile);
  fscanf(args,"%s\n %d\n",label,N_markers);

  if( (N_alleles = (int*) malloc((*N_markers)*sizeof(int))) == NULL) //N_alleles is a global pointer variable
    alloc_error("read_parameters with 'N_alleles'\n");

  fscanf(args,"%s\n",label);
  for(i=0;i<*N_markers;i++)
    {
      fscanf(args,"%d",&N_alleles[i]);
      if(i<*N_markers-1 && fgetc(args)!=',')
	{
	  fprintf(stderr,"ERROR:parameters file with allele number. Expected , after the allele number of the marker %d.\n\n",i+1);
	  exit(3);
	}
    }
  fscanf(args,"%s\n %d\n",label,N_c);
  fscanf(args,"%s\n %d\n",label,N_o);
  fscanf(args,"%s\n %lf\n",label,kappa);
  fscanf(args,"%s\n %lf\n",label,diff_exp);
  fscanf(args,"%s\n %d\n",label,M_clu);
  fscanf(args,"%s\n %d\n",label,allowed_jumps);

}

int **readPedigree(FILE *pedfile, int *n)
{

  /* 
     parents[i][0] = index of the father of individual i
     parents[i][1] = index of the mother of individual i
  */

  int **parents;
  int i,ind,sex,f,m;

  fscanf(pedfile,"%d\n",n);

  if((parents = (int**) malloc((*n)*sizeof(int*))) == NULL )
    alloc_error("readPedigree\n");       
  
  for(i=0; i<*n; i++)
    {
      if((parents[i] = (int*) malloc(2*sizeof(int))) == NULL )
	alloc_error("readPedigree\n");
      
      parents[i][0]=-1;
      parents[i][1]=-1;
      
    }
  
  for(i=0; i<*n && !(feof(pedfile)); i++)
    {
      fscanf(pedfile,"%d %d %d %d \n",&ind, &f, &m, &sex);
      if(ind>*n || ind <1 || f>*n || m>*n || f<0 || m<0)
	{
	  fprintf(stderr,"\nError: line %d in pedfile has invalid data, indexing out of bounds!\n\n",i+2); 
	  exit(3);
	}
      ind--; 
      if(parents[ind][0]!=-1 || parents[ind][1] != -1)
	{
	  fprintf(stderr,"\nError: individual indexed by %d in pedfile has multiple definitions!\n\n",ind+1); 
	  exit(3);
	}
      parents[ind][0]=f-1; parents[ind][1]=m-1;
    }

  if(i<*n)
    {
      fprintf(stderr,"\nError: Pedigree in pedfile contains only %d individuals\nthough the first line states that it contains %d!\n\n",i,*n);
      exit(3);
    }

  check_initial_pedigree(parents, *n);
  return parents;
}


int *read_sexes(FILE *fp)
{
  int *sexes;
  int i,ind,f,m,s;

  if( (sexes= (int*) malloc(N_individuals*sizeof(int)))==NULL)
    alloc_error("read_sexes");

  fscanf(fp,"%d",&i);
  for(i=0;i<N_individuals;i++)
    {
      fscanf(fp,"%d %d %d %d",&ind,&f,&m,&s);
      sexes[ind-1]=s;
    }

  return sexes;
}



void total_configuration(pair **genes, pair **origins, int **fixed_parents, COMP *C, double **afreq, int nonancestral)
{

  /* Makes genes into a total configuration if nonancestral = 1 
     (i.e. non ancestral alleles are sampled for founders and 
     dropped down through the pedigree.
     If nonancestral = 0, only the known alleles are considered 
     and other genotypes are left untyped.
  */

  int i,g,m,fa,mo,ind,a,j;
  double u,sum;

  g=N_levels-1;

  for(i=0;i<C[g].n;i++)
    {
      ind = C[g].individuals[i];

      for(m=0; m< N_markers;m++)
	{
	  for(j=0;j<2;j++)
	    {
	      if(genes[ind][m][j] == 0 && nonancestral)
		{
		  
		  u = 1.0*rand()/RAND_MAX;
		  a=0;
		  sum=0.0;
		  do
		    {
		      sum += afreq[m][a];
		      a++;
		    }while( sum < u && a < N_alleles[m]);

		  genes[ind][m][j] = a;
		} 
	    }
	}
    }

  for(g=N_levels-2; g>=0; g--)
    {
      for(i=0;i<C[g].n;i++)
	{
	  ind = C[g].individuals[i];
	  fa = fixed_parents[ind][0];
	  mo = fixed_parents[ind][1];

	  for(m=0; m< N_markers;m++)
	    {
	      for(j=0;j<2;j++)
		{
		  if(genes[ind][m][j] == 0 && nonancestral)
		    {
		      if(origins[ind][m][j] == 0)
			genes[ind][m][j] = (j==0) ? genes[fa][m][1] : genes[mo][m][1];
		      else
			genes[ind][m][j] = (j==0) ? genes[fa][m][0] : genes[mo][m][0];
		    }
		}
	    }
	}
    }

}


double single_kinship_with_arrays(int i,int j,int **fixed_parents,PARENTAL *parental,double ***kinship,double ***diff,int g,int diffg)
{

  /*  
      Returns the kinship coefficients of individuals i and j. 
      This is to be used only with the function count_kinship2.
  */

  double kin=0.0;
  int lev[2],founder0,founder1;

  if((kin=check_kinship_arrays(i,j,parental,kinship,diff,g,diffg))>=0.0)
    return kin;

  founder0=(fixed_parents[i][0]<0);
  founder1=(fixed_parents[j][0]<0);

  if(founder0 && founder1)
    {
      if(i==j)
	return 0.5;
      else
	return 0.0;
    }

  if(i==j) //neither is a founder
    return (0.5+0.5*single_kinship_with_arrays(fixed_parents[i][0],fixed_parents[i][1],fixed_parents,parental,kinship,diff,g,diffg));

  lev[0]=parental[i].levels[parental[i].nlevels-1];
  lev[1]=parental[j].levels[parental[j].nlevels-1];
  
  if((lev[1]>lev[0] && !founder0) || founder1)
    return 0.5*single_kinship_with_arrays(fixed_parents[i][0],j,fixed_parents,parental,kinship,diff,g,diffg)+0.5*single_kinship_with_arrays(fixed_parents[i][1],j,fixed_parents,parental,kinship,diff,g,diffg);
  else
    return 0.5*single_kinship_with_arrays(fixed_parents[j][0],i,fixed_parents,parental,kinship,diff,g,diffg)+0.5*single_kinship_with_arrays(fixed_parents[j][1],i,fixed_parents,parental,kinship,diff,g,diffg);

}


double ***transmission_probabilities(pair **genes,  int **fixed_parents, COMP *C, int l, int n_alleles, double *allelic_frequency)
{

  /* Counts the pseudo-probability for each individual. */

  int g,a,b,i,j,nzeros,f,m,gene[2];
  double ***conditionalF;

  /*conditionalF[i][a][n_alleles] shall contain the probability
    P(i transmits a) = 
     0.5*( conditionalF[i][a][a] + 
     sum_{b=0}^{n_alleles-1} conditionalF[i][a][n_alleles] )
  */

 if ((conditionalF=(double ***) malloc(N_individuals*sizeof(double **)))==NULL)
    alloc_error("transmission_probabilities\n");

  for(i=0; i<N_individuals; i++)
    {

      if ((conditionalF[i]=(double **) malloc(n_alleles*sizeof(double*)))==NULL)
	alloc_error("transmission_probabilities\n");
      
      for(j=0;j<n_alleles;j++)
	if ((conditionalF[i][j]=(double *) calloc((n_alleles+1),sizeof(double)))==NULL)
	   alloc_error("transmission_probabilities\n");
    }

 /*founder generation*/
  g=N_levels-1;

  for(j=0; j<C[g].n; j++)
    {
      i=C[g].individuals[j];
      nzeros=((genes[i][l][0]==0)+(genes[i][l][1]==0));
	  
      switch(nzeros)
	{
	case 0:

	  a=genes[i][l][0]-1;
	  b=genes[i][l][1]-1;

	  conditionalF[i][a][b] = conditionalF[i][b][a] = 1.0;
	  break;
	  
	case 1:

	  if (genes[i][l][0]>0) 
	    a=genes[i][l][0]-1;
	  else
	    a=genes[i][l][1]-1;

	  for( b=0;b<n_alleles;b++)
	      conditionalF[i][a][b] = conditionalF[i][b][a] = allelic_frequency[b];
	  break;
	  
	case 2: 

	  for(a=0;a<n_alleles;a++) 
	    for(b=0;b<n_alleles;b++)
	      {
		if(a != b)
		  conditionalF[i][a][b]=2*allelic_frequency[a]*allelic_frequency[b];
		else
		  conditionalF[i][a][b]=allelic_frequency[a]*allelic_frequency[b];
	      }
	  break;
	}

      for(a=0;a<n_alleles;a++)
	{ 
	  for(b=0;b<n_alleles;b++)
	    conditionalF[i][a][n_alleles] += conditionalF[i][a][b];

	  conditionalF[i][a][n_alleles] += conditionalF[i][a][a];
	  conditionalF[i][a][n_alleles] *= 0.5;
	}
    }


  for(g=N_levels-2; g>=0; g--)
    {
      for(j=0;j<C[g].n;j++)
	{
	  i=C[g].individuals[j];
	  
	  f=fixed_parents[i][0];
	  m=fixed_parents[i][1];
	  gene[0]=(genes[i][l][0] > 0 ) ? genes[i][l][0] : genes[i][l][1];
	  gene[1]=(genes[i][l][0] > 0 ) ? genes[i][l][1] : genes[i][l][0];
	  
	  nzeros=((gene[0]==0)+(gene[1]==0));
	  
	  switch(nzeros)
	    {
	      
	    case 0:

	      if((conditionalF[f][gene[0]-1][n_alleles]*conditionalF[m][gene[1]-1][n_alleles]==0) && 
		 (conditionalF[f][gene[1]-1][n_alleles]*conditionalF[m][gene[0]-1][n_alleles]==0))
		    {
		      printf("Inconsistent data in conditional probabilities at marker %d. \n Individual %d: (%d,%d).",l+1,i+1,gene[0],gene[1]); 
		      printf("Father %d: (%d,%d). Mother %d: (%d,%d).\n",f+1,genes[f][l][0],genes[f][l][1],m+1,genes[m][l][0],genes[m][l][1]);
		      exit(3);
		    }

	      conditionalF[i][gene[0]-1][gene[1]-1] = conditionalF[i][gene[1]-1][gene[0]-1] = 1.0;
	      break;
	      
	    case 1:

	      a=gene[0]-1;

	      if(conditionalF[f][a][n_alleles]+conditionalF[m][a][n_alleles]==0) // Does there exist at least one allele a among parents' genotypes?
		{
		  printf("Inconsistent data at marker %d. \n Individual %d: (%d,%d) cannot hold an allele %d.\n",l+1,i+1,gene[0],gene[1],a+1); 
		  printf("Father %d: (%d,%d). Mother %d: (%d,%d).\n",f+1,genes[f][l][0],genes[f][l][1],m+1,genes[m][l][0],genes[m][l][1]);
		  exit(3);
		}

	      for( b=0; b < n_alleles; b++) 
		{
		  if(b != a)
		    conditionalF[i][a][b] = conditionalF[f][a][n_alleles]*conditionalF[m][b][n_alleles]
		                            +conditionalF[f][b][n_alleles]*conditionalF[m][a][n_alleles];
		  else
		    conditionalF[i][a][b] = conditionalF[f][a][n_alleles]*conditionalF[m][b][n_alleles];

		  conditionalF[i][a][n_alleles] += conditionalF[i][a][b];
		}

	      conditionalF[i][a][n_alleles] += conditionalF[i][a][a];

	      for( b=0; b < n_alleles; b++) 
		{
		  conditionalF[i][a][b] /=  conditionalF[i][a][n_alleles]; //nonzero divisor (checked  above)
		  conditionalF[i][b][a] = conditionalF[i][a][b];
		}
	      conditionalF[i][a][a] *= 2.0;

	      break;

	    case 2:
	      for( a=0; a<n_alleles; a++) 
		{
		  for( b=0; b < a; b++)
		    conditionalF[i][a][b] = conditionalF[i][b][a] 
		      = conditionalF[f][a][n_alleles]*conditionalF[m][b][n_alleles]
		      +conditionalF[f][b][n_alleles]*conditionalF[m][a][n_alleles]; 

		  conditionalF[i][a][a] = conditionalF[f][a][n_alleles]*conditionalF[m][a][n_alleles];
		}
	      break;
	    }

	  for(a=0;a<n_alleles;a++)
	    {
	      conditionalF[i][a][n_alleles] = 0.0;
	      for(b=0;b<n_alleles;b++)
		conditionalF[i][a][n_alleles] += conditionalF[i][a][b];
	      conditionalF[i][a][n_alleles] += conditionalF[i][a][a];
	      conditionalF[i][a][n_alleles] *= 0.5;
	    }
	}
    }

  return conditionalF;

}












